const { parsers } = require("prettier/parser-babel");

/**
 * 词典序比较函数
 * @param {string} a - 第一个字符串
 * @param {string} b - 第二个字符串
 * @returns {number} - 比较结果
 */
function lexicalSort(a, b) {
  return a.localeCompare(b);
}

/**
 * 将值转换为 AST 节点
 * @param {*} value - 要转换的值
 * @returns {Object} - AST 节点
 */
function valueToAst(value) {
  if (typeof value === "string") {
    return { type: "StringLiteral", value };
  } else if (typeof value === "number") {
    return { type: "NumericLiteral", value };
  } else if (typeof value === "boolean") {
    return { type: "BooleanLiteral", value };
  } else if (value === null) {
    return { type: "NullLiteral" };
  } else if (Array.isArray(value)) {
    return {
      type: "ArrayExpression",
      elements: value.map(valueToAst),
    };
  } else if (typeof value === "object") {
    return {
      type: "ObjectExpression",
      properties: Object.entries(value).map(([key, val]) => ({
        type: "ObjectProperty",
        key: { type: "StringLiteral", value: key },
        value: valueToAst(val),
      })),
    };
  }
  throw new Error(`Unsupported value type: ${typeof value}`);
}

/**
 * 移除对象中的重复键，可选择是否按词典序排序
 * @param {*} obj - 输入对象
 * @param {Object} options - 配置选项
 * @param {boolean} options.enableSortJSON - 是否启用键排序
 * @returns {*} - 处理后的对象
 */
function removeDuplicateKeys(obj) {
  if (Array.isArray(obj)) {
    return obj.map(removeDuplicateKeys);
  }

  if (obj && typeof obj === "object") {
    const result = {};
    const keys = Object.keys(obj);

    // 从后向前遍历，保留最后出现的键值对
    for (let i = keys.length - 1; i >= 0; i--) {
      const key = keys[i];
      if (!(key in result)) {
        result[key] = removeDuplicateKeys(obj[key]);
      }
    }

    // 按字典序排序键
    const sortedKeys = Object.keys(result).sort((a, b) => a.localeCompare(b));
    const sortedResult = {};
    for (const key of sortedKeys) {
      sortedResult[key] = result[key];
    }

    return sortedResult;
  }

  return obj;
}

/**
 * 解析 JSON 文本
 * @param {string} text - JSON 文本
 * @param {Object} options - 配置选项
 * @returns {Object} - 解析后的对象
 * @throws {Error} - 解析错误
 */
function parseJSON(text) {
  try {
    const obj = JSON.parse(text);
    return removeDuplicateKeys(obj);
  } catch (error) {
    throw new Error(`Invalid JSON: ${error.message}`);
  }
}

// 获取原始的 estree parser
const originalEstreeParser = parsers.estree;

module.exports = {
  languages: [
    {
      name: "JSON",
      parsers: ["json-stringify"],
      extensions: [".json"],
      filenames: ["package.json", "composer.json"],
      linguistLanguageId: 174,
      vscodeLanguageIds: ["json", "jsonc"],
    },
  ],
  parsers: {
    "json-stringify": {
      ...parsers["json-stringify"],
      preprocess: (text, options) => {
        try {
          const obj = parseJSON(text);
          return JSON.stringify(obj, null, options.tabWidth || 2);
        } catch (error) {
          console.error("Error processing JSON:", error);
          return text;
        }
      },
      parse: (text, parsers, options) => {
        const ast = parsers["json-stringify"].parse(text);
        return ast;
      },
    },
  },
  printers: {
    estree: {
      print: (path, options, print) => {
        const node = path.getValue();
        if (node.type === "Program") {
          return print(path.get("body", 0, "expression")) + "\n";
        }
        if (node.type === "ObjectExpression") {
          const parts = ["{"];
          const props = path.map(print, "properties");
          if (props.length > 0) {
            parts.push("\n");
            parts.push(props.map((p) => "  " + p).join(",\n"));
            parts.push("\n");
          }
          parts.push("}");
          return parts.join("");
        }
        if (node.type === "ArrayExpression") {
          const parts = ["["];
          const elements = path.map(print, "elements");
          if (elements.length > 0) {
            parts.push("\n");
            parts.push(elements.map((e) => "  " + e).join(",\n"));
            parts.push("\n");
          }
          parts.push("]");
          return parts.join("");
        }
        if (node.type === "Property") {
          return [
            path.call(print, "key"),
            ": ",
            path.call(print, "value"),
          ].join("");
        }
        if (node.type === "Literal") {
          return node.raw;
        }
        return "";
      },
    },
  },
  options: {
    enableSortJSON: {
      type: "boolean",
      category: "JSON",
      default: false,
      description: "Sort JSON object keys in lexicographical order",
    },
  },
  defaultOptions: {
    tabWidth: 2,
    enableSortJSON: false,
  },
};
